@@ -67,6 +67,19 @@ class _raise_exception:
         return ([0], [1])
 
 
+class _raise_exception_2:
+    counter = 0
+
+    def fitness(self, dv):
+        if _raise_exception_2.counter == 300:
+            raise
+        _raise_exception_2.counter += 1
+        return [0]
+
+    def get_bounds(self):
+        return ([0], [1])
+
+
 class core_test_case(_ut.TestCase):
     """Test case for core PyGMO functionality.
 
@@ -392,9 +405,9 @@ class sga_test_case(_ut.TestCase):
         from .core import sga
         uda = sga()
         uda = sga(gen=1, cr=.90, eta_c=1., m=0.02, param_m=1., param_s=2, crossover="exponential",
-                  mutation="polynomial", selection="tournament", int_dim=0)
+                  mutation="polynomial", selection="tournament")
         uda = sga(gen=1, cr=.90, eta_c=1., m=0.02, param_m=1., param_s=2, crossover="exponential",
-                  mutation="polynomial", selection="tournament", int_dim=0, seed=32)
+                  mutation="polynomial", selection="tournament", seed=32)
         self.assertEqual(uda.get_seed(), 32)
         seed = uda.get_seed()
 
@@ -407,7 +420,7 @@ class nsga2_test_case(_ut.TestCase):
     def runTest(self):
         from .core import nsga2
         uda = nsga2()
-        uda = nsga2(gen=1, cr=0.95, eta_c=10, m=0.01, eta_m=10, int_dim=0)
+        uda = nsga2(gen=1, cr=0.95, eta_c=10, m=0.01, eta_m=10)
         uda = nsga2(gen=1, cr=0.95, eta_c=10, m=0.01,
                     eta_m=10, int_dim=0, seed=32)
         self.assertEqual(uda.get_seed(), 32)
@@ -831,6 +844,56 @@ class dtlz_test_case(_ut.TestCase):
         udp.p_distance(population(udp, 20))
 
 
+class minlp_rastrigin_test_case(_ut.TestCase):
+    """Test case for the MINLP Rastrigin
+
+    """
+
+    def runTest(self):
+        from .core import minlp_rastrigin, problem, population
+        udp = minlp_rastrigin(dim_c=2, dim_i=3)
+        prob = problem(udp)
+        self.assertTrue(prob.get_nx() == 5)
+        self.assertTrue(prob.get_nix() == 3)
+        self.assertTrue(prob.get_ncx() == 2)
+        pop = population(udp, 1)
+        self.assertTrue(int(pop.get_x()[0][-1]) == pop.get_x()[0][-1])
+        self.assertTrue(int(pop.get_x()[0][-2]) == pop.get_x()[0][-2])
+        self.assertTrue(int(pop.get_x()[0][-3]) == pop.get_x()[0][-3])
+        self.assertTrue(int(pop.get_x()[0][0]) != pop.get_x()[0][0])
+        self.assertTrue(int(pop.get_x()[0][1]) != pop.get_x()[0][1])
+
+
+class random_decision_vector_test_case(_ut.TestCase):
+    """Test case for random_decision_vector
+
+    """
+
+    def runTest(self):
+        from .core import random_decision_vector, set_global_rng_seed
+        set_global_rng_seed(42)
+        x = random_decision_vector(lb = [1.1,2.1,-3], ub = [2.1, 3.4,5], nix = 1)
+        self.assertTrue(int(x[-1]) == x[-1])
+        self.assertTrue(int(x[1]) != x[1])
+        set_global_rng_seed(42)
+        y = random_decision_vector(lb = [1.1,2.1,-3], ub = [2.1, 3.4,5], nix = 1)
+        self.assertTrue((x == y).all())
+        nan = float("nan")
+        inf = float("inf")
+        self.assertRaises(ValueError, lambda : random_decision_vector([1, 2], [0, 3]))
+        self.assertRaises(ValueError, lambda : random_decision_vector([1, -inf], [0, 32]))
+        self.assertRaises(ValueError, lambda : random_decision_vector([1, 2, 3], [0, 3]))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, 2, 3], [1, 4, nan]))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, 2, nan], [1, 4, 4]))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, nan, 3], [1, nan, 4]))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, 2, 3], [1, 4, 5], 4))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, 2, 3.1], [1, 4, 5], 1))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, 2, 3], [1, 4, 5.2], 1))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, -1.1, 3], [1, 2, 5], 2))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, -1.1, -inf], [1, 2, inf], 2))
+        self.assertRaises(ValueError, lambda : random_decision_vector([0, -1.1, inf], [1, 2, inf], 2))
+
+
 class luksan_vlcek1_test_case(_ut.TestCase):
     """Test case for the UDP Luksan Vlcek 1
 
@@ -1223,7 +1286,8 @@ class archipelago_test_case(_ut.TestCase):
         self.run_champions_tests()
         self.run_status_tests()
         if self._level > 0:
-            self.run_torture_test()
+            self.run_torture_test_0()
+            self.run_torture_test_1()
 
     def run_init_tests(self):
         from . import archipelago, de, rosenbrock, population, null_problem, thread_island, mp_island
@@ -1451,7 +1515,7 @@ class archipelago_test_case(_ut.TestCase):
         self.assertRaises(ValueError, lambda: a.wait_check())
         self.assertTrue(a.status == evolve_status.idle)
 
-    def run_torture_test(self):
+    def run_torture_test_0(self):
         from . import archipelago, de, ackley
 
         # pure C++
@@ -1476,6 +1540,31 @@ class archipelago_test_case(_ut.TestCase):
 
         self.assertRaises(BaseException, _)
 
+    def run_torture_test_1(self):
+        # A torture test inspired by the heisenbug detected by Dario on OSX.
+
+        from . import archipelago, sade, ackley
+
+        archi = archipelago(n=5, algo=sade(
+            50), prob=_raise_exception_2(), pop_size=20)
+        archi.evolve()
+        self.assertRaises(BaseException, lambda: archi.wait_check())
+
+        archi = archipelago(n=5, algo=sade(
+            50), prob=_raise_exception_2(), pop_size=20)
+        archi.evolve()
+        self.assertRaises(BaseException, lambda: archi.wait_check())
+        archi.wait_check()
+
+        archi = archipelago(n=1100, algo=sade(
+            500), prob=ackley(50), pop_size=50)
+        archi = archipelago(n=5, algo=sade(
+            50), prob=_raise_exception_2(), pop_size=20)
+        archi.evolve()
+        archi = archipelago(n=1100, algo=sade(
+            500), prob=ackley(50), pop_size=50)
+        archi.evolve()
+
 
 def run_test_suite(level=0):
     """Run the full test suite.
@@ -1498,8 +1587,8 @@ def run_test_suite(level=0):
     suite.addTest(_problem_test.problem_test_case())
     suite.addTest(_algorithm_test.algorithm_test_case())
     suite.addTest(_island_test.island_test_case())
-    suite.addTest(_island_test.mp_island_test_case())
-    suite.addTest(_island_test.ipyparallel_island_test_case())
+    suite.addTest(_island_test.mp_island_test_case(level))
+    suite.addTest(_island_test.ipyparallel_island_test_case(level))
     suite.addTest(pso_test_case())
     suite.addTest(bee_colony_test_case())
     suite.addTest(compass_search_test_case())
@@ -1515,6 +1604,7 @@ def run_test_suite(level=0):
     suite.addTest(global_rng_test_case())
     suite.addTest(estimate_sparsity_test_case())
     suite.addTest(estimate_gradient_test_case())
+    suite.addTest(random_decision_vector_test_case())
     try:
         from .core import cmaes
         suite.addTest(cmaes_test_case())
@@ -1525,6 +1615,7 @@ def run_test_suite(level=0):
     suite.addTest(cec2009_test_case())
     suite.addTest(cec2013_test_case())
     suite.addTest(luksan_vlcek1_test_case())
+    suite.addTest(minlp_rastrigin_test_case())
     suite.addTest(translate_test_case())
     suite.addTest(decompose_test_case())
     suite.addTest(unconstrain_test_case())
